---
elm:
  dependencies:
    vicramgon/logicus : latest
---


# Félix y el TX-150

> Fuente: Ejercios de Lógica Computacional
> (https://www.cs.us.es/~fsancho/?p=logica-informatica-2020-21)

Félix tiene en su garaje un taller que utiliza para la reparación de máquinas. Un buen amigo suyo va a hacerle una visita y obtiene la siguiente información:

- Todos los ordenadores son máquinas.

- El TX-150 es un ordenador

- Para cualquier máquina, Félix sabe sólo cómo estropearla o sólo cómo arreglarla.

- Toda máquina puede ser arreglada por alguien (incluso por otras máquinas), pero una máquina no puede arreglarse por sí misma.

- Las cosas sólo desesperan a quienes no son capaces de arreglarlas.

- Félix está deseperado con el TX-150.

Entonces, ¿puede arreglar Félix el TX-150?¿Puede estropearlo?

## Solución

Formalicemos el problema en el ámbito de la Lógica de Primer Orden. Para ello consideramos:

- Los símbolos de predicado: $O(x)$: "_$x$ es un ordenador_", $M(x)$: "_$x$ es una máquina_", $A(x,y)$: "_$x$ puede arreglar $y$_", $E(x,y)$: "_$x$ puede estropear $y$_" y $D(x, y)$: "_$x$ desespera a $y$_"

- Y los símbolos de constante $a$ y $b$ para representar al TX-150 y a Félix, respectivamente.

Antes de nada, vamos a importar el módulo de Sintaxis y Semántica en Primer Orden y algunos otros paquetes que nos harán falta para poder trabajar con las fórmulas.

Entonces las fórmulas asociadas a cada uno de los asertos corresponden a:

```elm {l id="imports"}

import LogicUS.FOL.SintaxSemantics exposing (..)
import LogicUS.FOL.SemanticTableaux exposing (..)
import Set exposing (Set)
import Dict exposing (Dict)

ffolRead : String -> FormulaFOL
ffolRead s = ffolReadExtraction <| ffolReadFromString s
```

$\boxed{\textit{Apartado 1}}$ Formalicemos los enunciados:


- Todos los ordenadores son máquinas.

    ```elm {l m=("$ " ++ (ffolToMathString psi1) ++ " $") context="1"  follows="imports"}
    psi1 : FormulaFOL
    psi1 = ffolRead "!A[x](O(x) -> M(x))"
    ```

- El TX-150 es un ordenador

    ```elm {l m=("$ " ++ (ffolToMathString psi2) ++ " $") context="1"}
    psi2 : FormulaFOL
    psi2 = ffolRead "O(*a)"
    ```

- Para cualquier máquina, Félix sabe sólo cómo estropearla o sólo cómo arreglarla.

    ```elm {l m=("$ " ++ (ffolToMathString psi3) ++ " $") context="1"}
    psi3 : FormulaFOL
    psi3 = ffolRead "!A[x](M(x) -> (A(*b, x) <-> ¬E(*b,x)))"
    ```

- Toda máquina puede ser arreglada por alguien (incluso por otras máquinas), pero una máquina no puede arreglarse por sí misma.

    ```elm {l m=("$ " ++ (ffolToMathString psi4) ++ " $") context="1"}
    psi4 : FormulaFOL
    psi4 = ffolRead "!A[x](M(x) -> (!E[y] A(y, x) & ¬A(x,x)))"
    ```

- Las cosas sólo desesperan a quienes no son capaces de arreglarlas.
    ```elm {l m=("$ " ++ (ffolToMathString psi5) ++ " $") context="1"}
    psi5 : FormulaFOL
    psi5 = ffolRead "!A[x]!A[y](D(x,y) <-> ¬A(y,x))"
    ```

- Félix está deseperado con el TX-150.
    ```elm {l m=("$ " ++ (ffolToMathString psi6) ++ " $") context="1"}
    psi6 : FormulaFOL
    psi6 = ffolRead "D(*a,*b)"
    ```

Ahora veámos si es posible que Félix pueda arreglar el TX-150. Esto es si el conjunto de las fórmulas anteriores junto con $A(b,a)$ es consistente. Probémos que no mediante tableros semánticos. 

```elm {l m=("$ " ++ (String.join "," <| List.map ffolToMathString sigma1) ++ " $") context="1"}
psi7 : FormulaFOL
psi7 = ffolRead "A(*b, *a)"

sigma1 : SetFOL
sigma1 = [ psi5, psi6, psi7]
```

Entonces construímos el tablero. Nótese que en realidad, las únicas fórmulas que son interesantes son las últimas tres.:

```elm {l r=(semanticTableauToDOT <| t1) context="1"}
t1 : FOLSemanticTableau
t1 = semanticTableau sigma1 2 5 -- |> semanticTableauRUNII
```

```dot
digraph G {
 rankdir=TB
 graph []
 node [shape=box, color=white, fontcolor=black]
 edge [dir=none, color=blue, fontcolor=blue]

 0 -> 1
 1 -> 2
 2 -> 3 [label=<γ(F₁,{x/a})>]
 3 -> 4 [label=<γ(F₄,{y/a})>]
 4 -> 5 [label=<α₁(F₅)>]
 5 -> 6 [label=<α₂(F₅)>]
 6 -> 7 [label=<β₁(F₆)>]
 6 -> 12 [label=<β₂(F₆)>]
 7 -> 8 [label=<β₁(F₇)>]
 7 -> 10 [label=<β₂(F₇)>]
 8 -> 9
 10 -> 11 [label=<⟂(F₈,F₉)>]
 12 -> 13 [label=<β₁(F₇)>]
 12 -> 15 [label=<β₂(F₇)>]
 13 -> 14 [label=<⟂(F₈,F₉)>]
 15 -> 16

 0 [label=<F₁≡∀x∀y(D(x,y)↔¬A(y,x))>]
 1 [label=<F₂≡D(a,b)>]
 2 [label=<F₃≡A(b,a)>]
 3 [label=<F₄≡∀y(D(a,y)↔¬A(y,a))>]
 4 [label=<F₅≡(D(a,a)↔¬A(a,a))>]
 5 [label=<F₆≡(D(a,a)→¬A(a,a))>]
 6 [label=<F₇≡(¬A(a,a)→D(a,a))>]
 7 [label=<F₈≡¬D(a,a)>]
 8 [label=<F₉≡A(a,a)>]
 9 [label=<❍>]
 10 [label=<F₉≡D(a,a)>]
 11 [label=<🗴>]
 12 [label=<F₈≡¬A(a,a)>]
 13 [label=<F₉≡A(a,a)>]
 14 [label=<🗴>]
 15 [label=<F₉≡D(a,a)>]
 16 [label=<⭘>]
}
```

De forma que el tablero es cerrado, por tanto Félix no puede arreglar el TX-150. Veamos ahora si se puede deducir que puede estropearlo.

```elm {l m=("$ " ++ (String.join "," <| List.map ffolToMathString sigma2) ++ " $") context="1"}
psi8 : FormulaFOL
psi8 = ffolRead "¬E(*b, *a)"

sigma2 : SetFOL
sigma2 = [psi1, psi2, psi3, psi4, psi5, psi6, psi8]
```

Entonces construímos el tablero y comprobamos que es cerrado:

```elm {l r=(t2IsClosed) context="1"}
t2IsClosed : Bool
t2IsClosed = semanticTableauIsInsat <| semanticTableau sigma2 2 100
```

Luego sí se tiene que $\sigma \models E(b, a)$, esto es que Félix puede estropearlo.